@@ -11,9 +11,9 @@ use Data::Dumper;
 use Protocol::DBus;
 use Protocol::DBus::Client;
 
-my $dbus = Protocol::DBus::Client::system();
+my $dbus = $> ? Protocol::DBus::Client::login_session() : Protocol::DBus::Client::system();
 
-$dbus->do_authn();
+$dbus->initialize();
 
 $dbus->get_message();
 
