@@ -16,7 +16,7 @@ waits for their responses, then ends:
 
     use experimental 'signatures';
 
-    my $dbus = Protocol::DBus::Client::AnyEvent::system();
+    my $dbus = Protocol::DBus::Client::AnyEvent::system()
 
     my $cv = AnyEvent->condvar();
 
@@ -86,55 +86,42 @@ sub _initialize {
     my $write_watch_r = \do { $self->{'_write_watch'} = undef };
 
     my $cb_r;
-
-    my $clear_watchers_cr = sub {
-        undef $$cb_r;
-        undef $$read_watch_r;
-        undef $$write_watch_r;
-    };
-
     my $cb = sub {
-        my $ok = eval {
-            if ( $dbus->initialize() ) {
-                $clear_watchers_cr->();
-                $y->();
-            }
-            else {
+        if ( $dbus->initialize() ) {
+            undef $$cb_r;
+            undef $$read_watch_r;
+            undef $$write_watch_r;
+            $y->();
+        }
+
+        # It seems unlikely that we’d need a write watch here.
+        # But just in case …
+        elsif ($dbus->init_pending_send()) {
+            $$write_watch_r ||= do {
+
                 my $current_sub = $$cb_r;
 
-                if ($dbus->init_pending_send()) {
-                    $$write_watch_r ||= AnyEvent->io(
-                        fh => $fileno,
-                        poll => 'w',
-                        cb => $current_sub,
-                    );
-                }
-                else {
-                    undef $$write_watch_r;
-
-                    $$read_watch_r ||= AnyEvent->io(
-                        fh => $fileno,
-                        poll => 'r',
-                        cb => $current_sub,
-                    );
-                }
-            }
-
-            1;
-        };
-
-        if (!$ok) {
-            $clear_watchers_cr->();
-            $n->($@);
+                AnyEvent->io(
+                    fh => $fileno,
+                    poll => 'w',
+                    cb => $current_sub,
+                );
+            };
+        }
+        else {
+            undef $$write_watch_r;
         }
     };
 
     $cb_r = \$cb;
 
-    # Postpone in order to give the caller time to set up promises, etc.
-    AnyEvent::postpone( sub { $cb->() } );
+    $$read_watch_r = AnyEvent->io(
+        fh => $fileno,
+        poll => 'r',
+        cb => $cb,
+    );
 
-    return;
+    $cb->();
 }
 
 sub _flush_send_queue {
@@ -158,42 +145,23 @@ sub _set_watches_and_create_messenger {
 
     my $fileno = $dbus->fileno();
 
-    my $write_watch_sr = \$self->{'_write_watch'};
-
-    my $flush_cr = sub {
-        _flush_send_queue( $dbus, $fileno, $write_watch_sr );
-    };
-
     if (!$self->{'_read_watch'}) {
 
-        $flush_cr->();
+        my $watch = undef;
+        _flush_send_queue( $dbus, $fileno, \$watch );
 
-        $self->{'_read_callback'} = $self->_create_get_message_callback();
+        $self->{'_send_watch_ref'} = \$watch;
 
-        $self->_resume();
-
-        my $read_watch_sr = \$self->{'_read_watch'};
-
-        $self->{'_stop_reading_cr'} = sub {
-            $$read_watch_sr = undef;
-        };
+        $self->{'_read_watch'} = AnyEvent->io(
+            fh => $fileno,
+            poll => 'r',
+            cb => $self->_create_get_message_callback(),
+        );
     }
 
-    return $flush_cr;
-}
+    my $watch_sr = $self->{'_send_watch_ref'};
 
-sub _pause {
-    $_[0]->{'_read_watch'} = undef;
-}
-
-sub _resume {
-    my ($self) = @_;
-
-    $self->{'_read_watch'} ||= AnyEvent->io(
-        fh => $self->{'db'}->fileno(),
-        poll => 'r',
-        cb => $self->{'_read_callback'},
-    );
+    return sub { _flush_send_queue( $dbus, $fileno, $watch_sr ) };
 }
 
 1;
